@@ -1,5 +1,5 @@
 /* lang.c -- language-dependent support.
-   $Id: lang.c,v 1.27 2007/07/01 21:20:32 karl Exp $
+   $Id: lang.c,v 1.30 2007/07/15 11:51:12 olegkat Exp $
 
    Copyright (C) 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007
    Free Software Foundation, Inc.
@@ -724,7 +724,8 @@ lang_transliterate_char (byte_t ch)
    new memory).  We use the same table as gettext, and return LL_CODE
    uppercased in the absence of any better possibility, with a warning.
    (gettext silently defaults to the C locale, but we want to give users
-   a shot at fixing ambiguities.)  */
+   a shot at fixing ambiguities.)  We also return en_US for en, while
+   gettext does not.  */
 
 #define SIZEOF(a) (sizeof(a) / sizeof(a[0]))
 
@@ -946,10 +947,10 @@ default_country_for_lang (const char *ll_code)
         }
     }
   
-  /* If didn't find one to copy, warn and duplicate.  */
+  /* If we didn't find one to copy, warn and duplicate.  */
   if (c == principal_len)
     {
-      if (strcasecmp (ll_code, "en"))
+      if (strcasecmp (ll_code, "en") == 0)
         cc_code = xstrdup ("en_US");
       else
         {
@@ -1157,7 +1158,7 @@ add_encoded_char (char *html_str, char *info_str)
     add_word_args ("&%s;", html_str);
   else if (xml)
     xml_insert_entity (html_str);
-  else if (enable_encoding)
+  else if (enable_encoding && document_encoding_code != no_encoding)
     {
       /* Look for HTML_STR in the current translation table.  */
       int rc = cm_search_iso_map (html_str);
@@ -1452,6 +1453,12 @@ cm_special_char (int arg)
         add_encoded_char ("aring", command);
       else if (strcmp (command, "ss") == 0)
         add_encoded_char ("szlig", command);
+      else if (strcmp (command, "guillemetleft") == 0
+               || strcmp (command, "guillemotleft") == 0)
+        add_encoded_char ("laquo", "<<");
+      else if (strcmp (command, "guillemetright") == 0
+               || strcmp (command, "guillemotright") == 0)
+        add_encoded_char ("raquo", ">>");
       else
         line_error ("cm_special_char internal error: command=@%s", command);
     }
